@@ -128,7 +128,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::onFlush()
     ocsd_datapath_resp_t resp = OCSD_RESP_CONT;
 
     // continue exception processing (can't go through processPacket as elements no longer on stack)
-    if(m_excep_proc != EXCEP_POP)
+    if(m_excep_info.proc != EXCEP_POP)
         resp = processException();
     // continue ongoing output operations on comitted elements.
     else if(m_curr_state == COMMIT_ELEM)
@@ -234,8 +234,9 @@ void TrcPktDecodeEtmV4I::resetDecoder()
     m_prev_overflow = false;
     m_P0_stack.delete_all();
     m_output_elem.init();
-    m_excep_proc = EXCEP_POP;
+    m_excep_info.proc = EXCEP_POP;
     m_flush_EOT = false;
+    m_last_IS = 0;
 }
 
 // this function can output an immediate generic element if this covers the complete packet decode, 
@@ -289,7 +290,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::decodePacket(bool &Complete)
 
     case ETM4_PKT_I_CTXT:
         {
-            if (m_P0_stack.createContextElem(m_curr_packet_in->getType(), m_index_curr_pkt, m_curr_packet_in->getContext()) == 0)
+            if (m_P0_stack.createContextElem(m_curr_packet_in->getType(), m_index_curr_pkt, m_curr_packet_in->getContext(), m_last_IS) == 0)
                 bAllocErr = true;
         }
         break;
@@ -299,7 +300,8 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::decodePacket(bool &Complete)
             etmv4_addr_val_t addr;
 
             addr.val = m_curr_packet_in->getAddrVal();
-            addr.isa = m_curr_packet_in->getAddrIS();
+            addr.isa = m_last_IS = m_curr_packet_in->getAddrIS();
+
             if (m_P0_stack.createAddrElem(m_curr_packet_in->getType(), m_index_curr_pkt, addr) == 0)
                 bAllocErr = true;
             is_addr = true;
@@ -311,7 +313,8 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::decodePacket(bool &Complete)
     case ETM4_PKT_I_ADDR_CTXT_L_32IS0:
     case ETM4_PKT_I_ADDR_CTXT_L_32IS1:    
         {
-            if (m_P0_stack.createContextElem(m_curr_packet_in->getType(), m_index_curr_pkt, m_curr_packet_in->getContext()) == 0)
+            m_last_IS = m_curr_packet_in->getAddrIS();
+            if (m_P0_stack.createContextElem(m_curr_packet_in->getType(), m_index_curr_pkt, m_curr_packet_in->getContext(),m_last_IS) == 0)
                 bAllocErr = true;
         }
     case ETM4_PKT_I_ADDR_L_32IS0:
@@ -324,7 +327,8 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::decodePacket(bool &Complete)
             etmv4_addr_val_t addr;
 
             addr.val = m_curr_packet_in->getAddrVal();
-            addr.isa = m_curr_packet_in->getAddrIS();
+            addr.isa = m_last_IS = m_curr_packet_in->getAddrIS();
+
             if (m_P0_stack.createAddrElem(m_curr_packet_in->getType(), m_index_curr_pkt, addr) == 0)
                 bAllocErr = true;
             is_addr = true;
@@ -507,6 +511,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::commitElements(bool &Complete)
     bool bPause = false;    // pause commit operation 
     bool bPopElem = true;       // do we remove the element from the stack (multi atom elements may need to stay!)
     int num_commit_req = m_P0_commit;
+    ocsd_trc_index_t err_idx = 0;
 
     Complete = true; // assume we exit due to completion of commit operation
 
@@ -517,8 +522,9 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::commitElements(bool &Complete)
         if(m_P0_stack.size() > 0)
         {
             pElem = m_P0_stack.back();  // get oldest element
-            
-            switch(pElem->getP0Type())
+            err_idx = pElem->getRootIndex(); // save index in case of error.
+
+            switch (pElem->getP0Type())
             {
             // indicates a trace restart - beginning of trace or discontinuiuty
             case P0_TRC_ON:
@@ -629,7 +635,7 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::commitElements(bool &Complete)
                 if ((resp = returnStackPop()) != OCSD_RESP_CONT)
                     break;
 
-                m_excep_proc = EXCEP_POP;   // set state in case we need to stop part way through
+                m_excep_info.proc = EXCEP_POP;   // set state in case we need to stop part way through
                 resp = processException();  // output trace + exception elements.
                 m_P0_commit--;
                 break;
@@ -661,10 +667,6 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::commitElements(bool &Complete)
         else
         {
             // too few elements for commit operation - decode error.
-            ocsd_trc_index_t err_idx = 0;
-            if(pElem)
-                err_idx = pElem->getRootIndex();
-              
             resp = OCSD_RESP_FATAL_INVALID_DATA;
             LogError(ocsdError(OCSD_ERR_SEV_ERROR,OCSD_ERR_COMMIT_PKT_OVERRUN,err_idx,m_CSID,"Not enough elements to commit"));
             bPause = true;
@@ -902,11 +904,13 @@ ocsd_datapath_resp_t TrcPktDecodeEtmV4I::processAtom(const ocsd_atm_val atom, bo
 ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
 {
     ocsd_datapath_resp_t resp = OCSD_RESP_CONT; 
-    bool excep_implied_P0 = false;          //!< exception implies P0
+    TrcStackElemExcept *pExceptElem;
 
-    if(m_excep_proc == EXCEP_POP)
+    m_excep_info.addr_b_tgt = false;
+    
+    if(m_excep_info.proc == EXCEP_POP)
     {
-        TrcStackElemExcept *pExceptElem = dynamic_cast<TrcStackElemExcept *>(m_P0_stack.back());  // get the exception element
+        pExceptElem = dynamic_cast<TrcStackElemExcept *>(m_P0_stack.back());  // get the exception element
         TrcStackElemAddr *pAddressElem = 0;
         TrcStackElemCtxt *pCtxtElem = 0;
         TrcStackElem *pElem = 0;
@@ -931,32 +935,52 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
             // extract address
             pAddressElem = static_cast<TrcStackElemAddr *>(pElem);
 
-            m_excep_addr = pAddressElem->getAddr();                
+            // fill in exception info for use later
+            m_excep_info.addr = pAddressElem->getAddr();                
+            m_excep_info.number = pExceptElem->getExcepNum();
+            m_excep_info.index = pExceptElem->getRootIndex();
+            m_excep_info.addr_b_tgt = pExceptElem->getPrevSame();
 
-            // if we have context, get that.
-            if(pCtxtElem)
-                 updateContext(pCtxtElem);
-
-            // record the exception number
-           m_excep_number = pExceptElem->getExcepNum();
-
-            // see if there is an implied P0 element on the exception.
-            excep_implied_P0 = pExceptElem->getPrevSame();
-
-            // save the trace index.
-            m_excep_index = pExceptElem->getRootIndex();
+            // see if there is an address + optional context element implied 
+            // prior to the exception.
+            if (m_excep_info.addr_b_tgt)
+            {
+                // this was a branch target address - update current setting
+                bool b64bit = m_instr_info.isa == ocsd_isa_aarch64;
+                if (pCtxtElem) {
+                    b64bit = pCtxtElem->getContext().SF;
+                }
+                m_instr_info.instr_addr = m_excep_info.addr.val;
+                m_instr_info.isa = (m_excep_info.addr.isa == 0) ? 
+                    (b64bit ? ocsd_isa_aarch64 : ocsd_isa_arm) : ocsd_isa_thumb2;
+                m_need_addr = false;
+            }
 
             // figure out next move
-            if(m_excep_addr.val == m_instr_info.instr_addr)
-                m_excep_proc = EXCEP_EXCEP;
+            if (pCtxtElem) {
+                m_excep_info.proc = EXCEP_CTXT;
+                updateContext(pCtxtElem);
+            }
+            else if(m_excep_info.addr.val == m_instr_info.instr_addr)
+                m_excep_info.proc = EXCEP_EXCEP;
             else
-                m_excep_proc = EXCEP_RANGE;
+                m_excep_info.proc = EXCEP_RANGE;
         }
         m_P0_stack.delete_popped();
     }
 
+    // output a context element
+    if (m_excep_info.proc == EXCEP_CTXT)
+    {
+        m_output_elem.setType(OCSD_GEN_TRC_ELEM_PE_CONTEXT);
+        resp = outputTraceElementIdx(m_excep_info.index, m_output_elem);
+        m_excep_info.proc = EXCEP_EXCEP;
+        if (!OCSD_DATA_RESP_IS_CONT(resp))
+            return resp;
+    }
+
     // output a range element
-    if(m_excep_proc == EXCEP_RANGE) 
+    if(m_excep_info.proc == EXCEP_RANGE) 
     {
         bool bWPFound = false;
         ocsd_err_t err;
@@ -964,8 +988,8 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
         // last instr_info address is the start address
         m_output_elem.st_addr = m_instr_info.instr_addr;
 
-        // look for either a WP or match to return address.
-        err = traceInstrToWP(bWPFound,!excep_implied_P0,m_excep_addr.val);
+        // look for match to return address.
+        err = traceInstrToWP(bWPFound,true,m_excep_info.addr.val);
 
         if(err != OCSD_OK)
         {
@@ -973,34 +997,21 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
             {
                 m_need_addr = true;
                 m_need_ctxt = true;
-                LogError(ocsdError(OCSD_ERR_SEV_WARN,err,m_excep_index,m_CSID,"Warning: unsupported instruction set processing exception packet."));  
+                LogError(ocsdError(OCSD_ERR_SEV_WARN,err,m_excep_info.index,m_CSID,"Warning: unsupported instruction set processing exception packet."));  
             }
             else
             {
                 resp = OCSD_RESP_FATAL_INVALID_DATA;
-                LogError(ocsdError(OCSD_ERR_SEV_ERROR,err,m_excep_index,m_CSID,"Error processing exception packet."));  
-                m_excep_proc = EXCEP_POP;  // nothing more to do, reset to start of exception handling
+                LogError(ocsdError(OCSD_ERR_SEV_ERROR,err,m_excep_info.index,m_CSID,"Error processing exception packet."));  
+                m_excep_info.proc = EXCEP_POP;  // nothing more to do, reset to start of exception handling
             }
         }
 
         if(bWPFound)
         {
-            // action according to waypoint type and atom value
-            if(excep_implied_P0)
-            {
-                switch(m_instr_info.type)
-                {
-                case OCSD_INSTR_BR:
-                m_instr_info.instr_addr = m_instr_info.branch_addr;
-                break;
-
-                case OCSD_INSTR_BR_INDIRECT:
-                m_instr_info.instr_addr = m_excep_addr.val;
-                break;
-                }
-            }
-            resp = outputTraceRange(true, m_excep_index);
-            m_excep_proc = EXCEP_EXCEP;
+            // waypoint address found - output range
+            resp = outputTraceRange(true, m_excep_info.index);
+            m_excep_info.proc = EXCEP_EXCEP;
         }
         else
         {
@@ -1010,32 +1021,33 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
             if(m_output_elem.st_addr != m_output_elem.en_addr)
             {
                 // some trace before we were out of memory access range
-                resp = outputTraceRange(true, m_excep_index);
+                resp = outputTraceRange(true, m_excep_info.index);
             }
 
-            m_excep_proc = m_mem_nacc_pending ? EXCEP_NACC : EXCEP_EXCEP;
+            m_excep_info.proc = m_mem_nacc_pending ? EXCEP_NACC : EXCEP_EXCEP;
         }
     }  
     
-    if((m_excep_proc == EXCEP_NACC) && OCSD_DATA_RESP_IS_CONT(resp))
+    if((m_excep_info.proc == EXCEP_NACC) && OCSD_DATA_RESP_IS_CONT(resp))
     {
         m_output_elem.setType(OCSD_GEN_TRC_ELEM_ADDR_NACC);
         m_output_elem.st_addr = m_nacc_addr;
-        resp = outputTraceElementIdx(m_excep_index,m_output_elem);
-        m_excep_proc = EXCEP_EXCEP;
+        resp = outputTraceElementIdx(m_excep_info.index,m_output_elem);
+        m_excep_info.proc = EXCEP_EXCEP;
         m_mem_nacc_pending = false;
     }
     
-    if((m_excep_proc == EXCEP_EXCEP) && OCSD_DATA_RESP_IS_CONT(resp))
+    if((m_excep_info.proc == EXCEP_EXCEP) && OCSD_DATA_RESP_IS_CONT(resp))
     {
         // output element.
         m_output_elem.setType(OCSD_GEN_TRC_ELEM_EXCEPTION);
         // add end address as preferred return address to end addr in element
-        m_output_elem.en_addr = m_excep_addr.val;
+        m_output_elem.en_addr = m_excep_info.addr.val;
         m_output_elem.excep_ret_addr = 1;
-        m_output_elem.exception_number = m_excep_number;
-        resp = outputTraceElementIdx(m_excep_index,m_output_elem);  
-        m_excep_proc = EXCEP_POP;
+        m_output_elem.excep_ret_addr_br_tgt = m_excep_info.addr_b_tgt;
+        m_output_elem.exception_number = m_excep_info.number;
+        resp = outputTraceElementIdx(m_excep_info.index,m_output_elem);  
+        m_excep_info.proc = EXCEP_POP;
     }   
     return resp;
 }
@@ -1043,10 +1055,7 @@ ocsd_datapath_resp_t  TrcPktDecodeEtmV4I::processException()
 void TrcPktDecodeEtmV4I::SetInstrInfoInAddrISA(const ocsd_vaddr_t addr_val, const uint8_t isa)
 {
     m_instr_info.instr_addr = addr_val;
-    if(m_is_64bit)
-        m_instr_info.isa = ocsd_isa_aarch64;
-    else
-        m_instr_info.isa = (isa == 0) ? ocsd_isa_arm : ocsd_isa_thumb2;
+    m_instr_info.isa = calcISA(m_is_64bit, isa);
 }
 
 // trace an instruction range to a waypoint - and set next address to restart from.
@@ -1120,6 +1129,9 @@ void TrcPktDecodeEtmV4I::updateContext(TrcStackElemCtxt *pCtxtElem)
         m_output_elem.context.vmid_valid = 1;
         m_vmid_id = m_output_elem.context.vmid = ctxt.VMID;
     }
+
+    // need to update ISA in case context follows address.
+    m_output_elem.isa = m_instr_info.isa = calcISA(m_is_64bit, pCtxtElem->getIS());
     m_need_ctxt = false;
 }
 
